@@ -357,6 +357,7 @@ perform_member_init (tree member, tree init)
 	{
 	  /* Initialization of one array from another.  */
 	  finish_expr_stmt (build_vec_init (decl, NULL_TREE, TREE_VALUE (init),
+					    /*explicit_default_init_p=*/false,
 					    /* from_array=*/1));
 	}
       else
@@ -1017,11 +1018,11 @@ expand_member_init (tree name)
       if (!direct_binfo && !virtual_binfo)
 	{
 	  if (CLASSTYPE_VBASECLASSES (current_class_type))
-	    error ("type %qD is not a direct or virtual base of %qT",
-		   name, current_class_type);
+	    error ("type %qT is not a direct or virtual base of %qT",
+		   basetype, current_class_type);
 	  else
-	    error ("type %qD is not a direct base of %qT",
-		   name, current_class_type);
+	    error ("type %qT is not a direct base of %qT",
+		   basetype, current_class_type);
 	  return NULL_TREE;
 	}
 
@@ -1111,6 +1112,7 @@ build_aggr_init (tree exp, tree init, int flags)
       if (itype && cp_type_quals (itype) != TYPE_UNQUALIFIED)
 	itype = TREE_TYPE (init) = TYPE_MAIN_VARIANT (itype);
       stmt_expr = build_vec_init (exp, NULL_TREE, init,
+				  /*explicit_default_init_p=*/false,
 				  itype && same_type_p (itype,
 							TREE_TYPE (exp)));
       TREE_READONLY (exp) = was_const;
@@ -1390,9 +1392,10 @@ build_offset_ref (tree type, tree name, bool address_p)
 
   if (TREE_CODE (name) == BIT_NOT_EXPR)
     {
+      name = TREE_OPERAND (name, 0);
       if (! check_dtor_name (type, name))
 	error ("qualified type %qT does not match destructor name %<~%T%>",
-		  type, TREE_OPERAND (name, 0));
+	       type, name);
       name = dtor_identifier;
     }
 
@@ -1556,49 +1559,87 @@ build_offset_ref (tree type, tree name, bool address_p)
   return member;
 }
 
-/* If DECL is a CONST_DECL, or a constant VAR_DECL initialized by
-   constant of integral or enumeration type, then return that value.
-   These are those variables permitted in constant expressions by
-   [5.19/1].  FIXME:If we did lazy folding, this could be localized.  */
+/* If DECL is a scalar enumeration constant or variable with a
+   constant initializer, return the initializer (or, its initializers,
+   recursively); otherwise, return DECL.  If INTEGRAL_P, the
+   initializer is only returned if DECL is an integral
+   constant-expression.  */
 
-tree
-integral_constant_value (tree decl)
+static tree
+constant_value_1 (tree decl, bool integral_p)
 {
-  while ((TREE_CODE (decl) == CONST_DECL
-	  || (TREE_CODE (decl) == VAR_DECL
-	      /* And so are variables with a 'const' type -- unless they
-		 are also 'volatile'.  */
-	      && CP_TYPE_CONST_NON_VOLATILE_P (TREE_TYPE (decl))
-	      && DECL_INITIALIZED_BY_CONSTANT_EXPRESSION_P (decl)))
-	 && DECL_INITIAL (decl)
-	 && DECL_INITIAL (decl) != error_mark_node
-	 && TREE_TYPE (DECL_INITIAL (decl))
-	 && INTEGRAL_OR_ENUMERATION_TYPE_P (TREE_TYPE (DECL_INITIAL (decl))))
-    decl = DECL_INITIAL (decl);
+  while (TREE_CODE (decl) == CONST_DECL
+	 || (integral_p 
+	     ? DECL_INTEGRAL_CONSTANT_VAR_P (decl)
+	     : (TREE_CODE (decl) == VAR_DECL
+		&& CP_TYPE_CONST_NON_VOLATILE_P (TREE_TYPE (decl)))))
+    {
+      tree init;
+      /* Static data members in template classes may have
+	 non-dependent initializers.  References to such non-static
+	 data members are not value-dependent, so we must retrieve the
+	 initializer here.  The DECL_INITIAL will have the right type,
+	 but will not have been folded because that would prevent us
+	 from performing all appropriate semantic checks at
+	 instantiation time.  */
+      if (DECL_CLASS_SCOPE_P (decl)
+	  && CLASSTYPE_TEMPLATE_INFO (DECL_CONTEXT (decl))
+	  && uses_template_parms (CLASSTYPE_TI_ARGS 
+				  (DECL_CONTEXT (decl))))
+	{
+	  ++processing_template_decl;
+	  init = fold_non_dependent_expr (DECL_INITIAL (decl));
+	  --processing_template_decl;
+	}
+      else
+	{
+	  /* If DECL is a static data member in a template
+	     specialization, we must instantiate it here.  The
+	     initializer for the static data member is not processed
+	     until needed; we need it now.  */
+	  mark_used (decl);
+	  init = DECL_INITIAL (decl);
+	}
+      if (init == error_mark_node)
+	return error_mark_node;
+      if (!init
+	  || !TREE_TYPE (init)
+	  || (integral_p
+	      ? !INTEGRAL_OR_ENUMERATION_TYPE_P (TREE_TYPE (init))
+	      : (!TREE_CONSTANT (init)
+		 /* Do not return an aggregate constant (of which
+		    string literals are a special case), as we do not
+		    want to make inadvertant copies of such entities,
+		    and we must be sure that their addresses are the
+		    same everywhere.  */
+		 || TREE_CODE (init) == CONSTRUCTOR
+		 || TREE_CODE (init) == STRING_CST)))
+	break;
+      decl = unshare_expr (init);
+    }
   return decl;
 }
 
-/* A more relaxed version of integral_constant_value, for which type
-   is not considered.  This is used by the common C/C++ code, and not
-   directly by the C++ front end.  */
+/* If DECL is a CONST_DECL, or a constant VAR_DECL initialized by
+   constant of integral or enumeration type, then return that value.
+   These are those variables permitted in constant expressions by
+   [5.19/1].  */
+  
+tree
+integral_constant_value (tree decl)
+  {
+  return constant_value_1 (decl, /*integral_p=*/true);
+}
+  
+/* A more relaxed version of integral_constant_value, used by the
+   common C/C++ code and by the C++ front-end for optimization
+   purposes.  */
 
 tree
 decl_constant_value (tree decl)
 {
-  if ((TREE_CODE (decl) == CONST_DECL
-      || (TREE_CODE (decl) == VAR_DECL
-	  /* And so are variables with a 'const' type -- unless they
-             are also 'volatile'.  */
-	  && CP_TYPE_CONST_NON_VOLATILE_P (TREE_TYPE (decl))))
-      && DECL_INITIAL (decl)
-      && DECL_INITIAL (decl) != error_mark_node
-      /* This is invalid if initial value is not constant.  If it has
-       	 either a function call, a memory reference, or a variable,
-       	 then re-evaluating it could give different results.  */
-      && TREE_CONSTANT (DECL_INITIAL (decl)))
-    return DECL_INITIAL (decl);
-  
-  return decl;
+  return constant_value_1 (decl, 
+			   /*integral_p=*/processing_template_decl);
 }
 
 /* Common subroutines of build_new and build_vec_delete.  */
@@ -1822,6 +1863,9 @@ build_new_1 (tree exp)
 	}
     }
 
+  if (!complete_type_or_else (type, exp))
+    return error_mark_node;
+
   /* If our base type is an array, then make sure we know how many elements
      it has.  */
   for (elt_type = type;
@@ -1830,9 +1874,6 @@ build_new_1 (tree exp)
     nelts = cp_build_binary_op (MULT_EXPR, nelts, 
 				array_type_nelts_top (elt_type));
 
-  if (!complete_type_or_else (elt_type, exp))
-    return error_mark_node;
-
   if (TREE_CODE (elt_type) == VOID_TYPE)
     {
       error ("invalid type %<void%> for new");
@@ -1917,6 +1958,11 @@ build_new_1 (tree exp)
 	  args = tree_cons (NULL_TREE, size, placement);
 	  /* Do name-lookup to find the appropriate operator.  */
 	  fns = lookup_fnfields (elt_type, fnname, /*protect=*/2);
+	  if (fns == NULL_TREE)
+	    {
+	      error ("no suitable %qD found in class %qT", fnname, elt_type);
+	      return error_mark_node;
+	    }
 	  if (TREE_CODE (fns) == TREE_LIST)
 	    {
 	      error ("request for member %qD is ambiguous", fnname);
@@ -2049,46 +2095,60 @@ build_new_1 (tree exp)
 
       init_expr = build_indirect_ref (data_addr, NULL);
 
-      if (init == void_zero_node)
-	init = build_default_init (full_type, nelts);
-      else if (init && array_p)
-	pedwarn ("ISO C++ forbids initialization in array new");
-
       if (array_p)
 	{
+	  bool explicit_default_init_p = false;
+
+	  if (init == void_zero_node)
+	    {
+	      init = NULL_TREE;
+	      explicit_default_init_p = true;
+	    }
+	  else if (init)
+	    pedwarn ("ISO C++ forbids initialization in array new");
+
 	  init_expr
 	    = build_vec_init (init_expr,
 			      cp_build_binary_op (MINUS_EXPR, outer_nelts,
 						  integer_one_node),
-			      init, /*from_array=*/0);
+			      init, 
+			      explicit_default_init_p,
+			      /*from_array=*/0);
 
 	  /* An array initialization is stable because the initialization
 	     of each element is a full-expression, so the temporaries don't
 	     leak out.  */
 	  stable = true;
 	}
-      else if (TYPE_NEEDS_CONSTRUCTING (type))
-	{
-	  init_expr = build_special_member_call (init_expr, 
-						 complete_ctor_identifier,
-						 init, elt_type,
-						 LOOKUP_NORMAL);
-	  stable = stabilize_init (init_expr, &init_preeval_expr);
-	}
       else
 	{
-	  /* We are processing something like `new int (10)', which
-	     means allocate an int, and initialize it with 10.  */
-
-	  if (TREE_CODE (init) == TREE_LIST)
-	    init = build_x_compound_expr_from_list (init, "new initializer");
+	  if (init == void_zero_node)
+	    init = build_default_init (full_type, nelts);
 
+	  if (TYPE_NEEDS_CONSTRUCTING (type))
+	    {
+	      init_expr = build_special_member_call (init_expr, 
+						     complete_ctor_identifier,
+						     init, elt_type,
+						     LOOKUP_NORMAL);
+	      stable = stabilize_init (init_expr, &init_preeval_expr);
+	    }
 	  else
-	    gcc_assert (TREE_CODE (init) != CONSTRUCTOR
-			|| TREE_TYPE (init) != NULL_TREE);
+	    {
+	      /* We are processing something like `new int (10)', which
+		 means allocate an int, and initialize it with 10.  */
 
-	  init_expr = build_modify_expr (init_expr, INIT_EXPR, init);
-	  stable = stabilize_init (init_expr, &init_preeval_expr);
+	      if (TREE_CODE (init) == TREE_LIST)
+		init = build_x_compound_expr_from_list (init, 
+							"new initializer");
+
+	      else
+		gcc_assert (TREE_CODE (init) != CONSTRUCTOR
+			    || TREE_TYPE (init) != NULL_TREE);
+
+	      init_expr = build_modify_expr (init_expr, INIT_EXPR, init);
+	      stable = stabilize_init (init_expr, &init_preeval_expr);
+	    }
 	}
 
       if (init_expr == error_mark_node)
@@ -2193,8 +2253,7 @@ build_new_1 (tree exp)
   rval = build_nop (pointer_type, rval);
 
   /* A new-expression is never an lvalue.  */
-  if (real_lvalue_p (rval))
-    rval = build1 (NON_LVALUE_EXPR, TREE_TYPE (rval), rval);
+  rval = rvalue (rval);
 
   return rval;
 }
@@ -2369,8 +2428,12 @@ get_temp_regvar (tree type, tree init)
    MAXINDEX is the maximum index of the array (one less than the
      number of elements).  It is only used if
      TYPE_DOMAIN (TREE_TYPE (BASE)) == NULL_TREE.
+
    INIT is the (possibly NULL) initializer.
 
+   If EXPLICIT_DEFAULT_INIT_P is true, then INIT must be NULL.  All
+   elements in the array are default-initialized.
+
    FROM_ARRAY is 0 if we should init everything with INIT
    (i.e., every element initialized from INIT).
    FROM_ARRAY is 1 if we should index into INIT in parallel
@@ -2379,7 +2442,9 @@ get_temp_regvar (tree type, tree init)
    but use assignment instead of initialization.  */
 
 tree
-build_vec_init (tree base, tree maxindex, tree init, int from_array)
+build_vec_init (tree base, tree maxindex, tree init, 
+		bool explicit_default_init_p,
+		int from_array)
 {
   tree rval;
   tree base2 = NULL_TREE;
@@ -2408,6 +2473,9 @@ build_vec_init (tree base, tree maxindex, tree init, int from_array)
   if (maxindex == NULL_TREE || maxindex == error_mark_node)
     return error_mark_node;
 
+  if (explicit_default_init_p)
+    gcc_assert (!init);
+
   inner_elt_type = strip_array_types (atype);
   if (init
       && (from_array == 2
@@ -2539,7 +2607,7 @@ build_vec_init (tree base, tree maxindex, tree init, int from_array)
      We do need to keep going if we're copying an array.  */
 
   if (from_array
-      || (TYPE_NEEDS_CONSTRUCTING (type)
+      || ((TYPE_NEEDS_CONSTRUCTING (type) || explicit_default_init_p)
 	  && ! (host_integerp (maxindex, 0)
 		&& (num_initialized_elts
 		    == tree_low_cst (maxindex, 0) + 1))))
@@ -2548,6 +2616,7 @@ build_vec_init (tree base, tree maxindex, tree init, int from_array)
 	 we've already initialized all the elements.  */
       tree for_stmt;
       tree elt_init;
+      tree to;
 
       for_stmt = begin_for_stmt ();
       finish_for_init_stmt (for_stmt);
@@ -2557,9 +2626,10 @@ build_vec_init (tree base, tree maxindex, tree init, int from_array)
       finish_for_expr (build_unary_op (PREDECREMENT_EXPR, iterator, 0),
 		       for_stmt);
 
+      to = build1 (INDIRECT_REF, type, base);
+
       if (from_array)
 	{
-	  tree to = build1 (INDIRECT_REF, type, base);
 	  tree from;
 
 	  if (base2)
@@ -2582,8 +2652,15 @@ build_vec_init (tree base, tree maxindex, tree init, int from_array)
 	    sorry
 	      ("cannot initialize multi-dimensional array with initializer");
 	  elt_init = build_vec_init (build1 (INDIRECT_REF, type, base),
-				     0, 0, 0);
+				     0, 0, 
+				     /*explcit_default_init_p=*/false,
+				     0);
 	}
+      else if (!TYPE_NEEDS_CONSTRUCTING (type))
+	elt_init = (build_modify_expr 
+		    (to, INIT_EXPR,
+		     build_zero_init (type, size_one_node,
+				      /*static_storage_p=*/false)));
       else
 	elt_init = build_aggr_init (build1 (INDIRECT_REF, type, base), 
 				    init, 0);
