@@ -605,20 +605,19 @@ find_substitution (tree node)
 				       SUBID_CHAR_TRAITS))
 	{
 	  /* Got them.  Is this basic_istream?  */
-	  tree name = DECL_NAME (CLASSTYPE_TI_TEMPLATE (type));
-	  if (name == subst_identifiers[SUBID_BASIC_ISTREAM])
+	  if (is_std_substitution (decl, SUBID_BASIC_ISTREAM))
 	    {
 	      write_string ("Si");
 	      return 1;
 	    }
 	  /* Or basic_ostream?  */
-	  else if (name == subst_identifiers[SUBID_BASIC_OSTREAM])
+	  else if (is_std_substitution (decl, SUBID_BASIC_OSTREAM))
 	    {
 	      write_string ("So");
 	      return 1;
 	    }
 	  /* Or basic_iostream?  */
-	  else if (name == subst_identifiers[SUBID_BASIC_IOSTREAM])
+	  else if (is_std_substitution (decl, SUBID_BASIC_IOSTREAM))
 	    {
 	      write_string ("Sd");
 	      return 1;
