@@ -113,7 +113,7 @@ init_rtti_processing (void)
   
   push_namespace (std_identifier);
   type_info_type = xref_tag (class_type, get_identifier ("type_info"),
-			     /*tag_scope=*/ts_global, false);
+			     /*tag_scope=*/ts_current, false);
   pop_namespace ();
   const_type_info_type_node
     = build_qualified_type (type_info_type, TYPE_QUAL_CONST);
@@ -527,7 +527,7 @@ build_dynamic_cast_1 (tree type, tree expr)
 	expr = build_base_path (PLUS_EXPR, convert_from_reference (expr),
 				binfo, 0);
 	if (TREE_CODE (exprtype) == POINTER_TYPE)
-	  expr = non_lvalue (expr);
+	  expr = rvalue (expr);
 	return expr;
       }
   }
@@ -624,7 +624,7 @@ build_dynamic_cast_1 (tree type, tree expr)
 	      push_nested_namespace (ns);
 	      tinfo_ptr = xref_tag (class_type,
 				    get_identifier ("__class_type_info"),
-				    /*tag_scope=*/ts_global, false);
+				    /*tag_scope=*/ts_current, false);
 	      
 	      tinfo_ptr = build_pointer_type
 		(build_qualified_type
@@ -805,7 +805,7 @@ tinfo_base_init (tree desc, tree target)
   
       push_nested_namespace (abi_node);
       real_type = xref_tag (class_type, TINFO_REAL_NAME (desc),
-			    /*tag_scope=*/ts_global, false);
+			    /*tag_scope=*/ts_current, false);
       pop_nested_namespace (abi_node);
   
       if (!COMPLETE_TYPE_P (real_type))
@@ -1337,7 +1337,7 @@ emit_support_tinfos (void)
   push_nested_namespace (abi_node);
   bltn_type = xref_tag (class_type,
 			get_identifier ("__fundamental_type_info"), 
-			/*tag_scope=*/ts_global, false);
+			/*tag_scope=*/ts_current, false);
   pop_nested_namespace (abi_node);
   if (!COMPLETE_TYPE_P (bltn_type))
     return;
@@ -1426,7 +1426,8 @@ emit_tinfo_decl (tree decl)
       var_init = get_pseudo_ti_init (type, var_desc);
       DECL_INITIAL (decl) = var_init;
       mark_used (decl);
-      cp_finish_decl (decl, var_init, NULL_TREE, 0);
+      cp_finish_decl (decl, var_init, /*init_const_expr_p=*/false, 
+		      NULL_TREE, 0);
       return true;
     }
   else
